/*
************************************************************************************************************************
*                                                         eGON
*                                         the Embedded GO-ON Bootloader System
*
*                             Copyright(C), 2006-2008, SoftWinners Microelectronic Co., Ltd.
*											       All Rights Reserved
*
* File Name : check.c
*
* Author : Gary.Wang
*
* Version : 1.1.0
*
* Date : 2007.10.12
*
* Description : This file provides a function to check Boot0 and Boot1.
*
* Others : None at present.
*
*
* History :
*
*  <Author>        <time>       <version>      <description>
*
* Gary.Wang       2007.10.12      1.1.0        build the file
*
************************************************************************************************************************
*/

#include  "check_sum.h"

typedef struct tag_CRC32_DATA
{
	unsigned int CRC;				//intĴС32λ
	unsigned int CRC_32_Tbl[256];	//
}CRC32_DATA_t;
//#pragma arm section  code="check_magic"
/********************************************************************************
*: check_magic
*ԭ: __s32 check_magic( __u32 *mem_base, const char *magic )
*: ʹá͡Уڴеһ
*ڲ: mem_base       Bootļڴеʼַ
*          magic          Bootmagic
*  ֵ: CHECK_IS_CORRECT      Уȷ
*          CHECK_IS_WRONG        У
*    ע:
********************************************************************************/
__s32 check_magic( __u32 *mem_base, const char *magic )
{
	__u32 i;
	boot_file_head_t *bfh;
	__u32 sz;
	unsigned char *p;


	bfh = (boot_file_head_t *)mem_base;
	p = bfh->magic;
	for( i = 0, sz = sizeof( bfh->magic );  i < sz;  i++ )
	{
		if( *p++ != *magic++ )
			return CHECK_IS_WRONG;
	}


	return CHECK_IS_CORRECT;
}

//#pragma arm section




//#pragma arm section  code="check_sum"
/********************************************************************************
*: check_sum
*ԭ: __s32 check_sum( __u32 *mem_base, __u32 size, const char *magic )
*: ʹá͡Уڴеһ
*ڲ: mem_base           Уڴеʼַ4ֽڶģ
*          size               УݵĸֽΪλ4ֽڶģ
*  ֵ: CHECK_IS_CORRECT   Уȷ
*          CHECK_IS_WRONG     У
*    ע:
********************************************************************************/
__s32 check_sum( __u32 *mem_base, __u32 size )
{
	__u32 *buf;
	__u32 count;
	__u32 src_sum;
	__u32 sum;
	boot_file_head_t  *bfh;


	bfh = (boot_file_head_t *)mem_base;

	/* У */
	src_sum = bfh->check_sum;                  // Boot_file_headеġcheck_sumֶȡУ
	bfh->check_sum = STAMP_VALUE;              // STAMP_VALUEдBoot_file_headеġcheck_sumֶ

	count = size >> 2;                         //  ֣4bytesΪλ
	sum = 0;
	buf = (__u32 *)mem_base;
	do
	{
		sum += *buf++;                         // ۼӣУ
		sum += *buf++;                         // ۼӣУ
		sum += *buf++;                         // ۼӣУ
		sum += *buf++;                         // ۼӣУ
	}while( ( count -= 4 ) > (4-1) );

	while( count-- > 0 )
		sum += *buf++;

	bfh->check_sum = src_sum;                  // ָBoot_file_headеġcheck_sumֶεֵ

	if( sum == src_sum )
		return CHECK_IS_CORRECT;               // Уɹ
	else
		return CHECK_IS_WRONG;                 // Уʧ
}

//#pragma arm section

#define  ADD_SUM_MAGIC	0x62332672

__s32 verify_sum( void *mem_base, __u32 size )
{
	__u32 *buf;
	__u32 count;
	__u32 sum;

	count = size >> 2;                         //  ֣4bytesΪλ
	sum = 0;
	buf = (__u32 *)mem_base;
	while(count--)
	{
		sum += *buf++;                         // ۼӣУ
	};

	switch(size & 0x03)
	{
		case 0:
			return sum;
		case 1:
			sum += (*buf & 0x000000ff);
			break;
		case 2:
			sum += (*buf & 0x0000ffff);
			break;
		case 3:
			sum += (*buf & 0x00ffffff);
			break;
	}

	return sum;
}
//#pragma arm section  code="check_file"
/********************************************************************************
*: check_file
*ԭ: __s32 check_file( __u32 *mem_base, __u32 size, const char *magic )
*: ʹá͡Уڴеһ
*ڲ: mem_base       Уڴеʼַ4ֽڶģ
*          size           УݵĸֽΪλ4ֽڶģ
*          magic          magic number, Уļıʶ
*  ֵ: CHECK_IS_CORRECT       Уȷ
*          CHECK_IS_WRONG         У
*    ע:
********************************************************************************/
__s32 check_file( __u32 *mem_base, __u32 size, const char *magic )
{
	if( check_magic( mem_base, magic ) == CHECK_IS_CORRECT
        &&check_sum( mem_base, size  ) == CHECK_IS_CORRECT )
        return CHECK_IS_CORRECT;
    else
    	return CHECK_IS_WRONG;
}
/*
************************************************************************************************************
*
*                                             _calc_crc32
*
*    ƣ
*
*    б
*
*    ֵ  
*
*    ˵    
*
*
************************************************************************************************************
*/
__u32 calc_crc32(void * buffer, __u32 length)
{
	__u32 i, j;
	CRC32_DATA_t crc32;		//
	__u32 CRC32 = 0xffffffff; //óʼֵ

	crc32.CRC = 0;

	for( i = 0; i < 256; ++i)//++iЧ
	{
		crc32.CRC = i;
		for( j = 0; j < 8 ; ++j)
		{
			//ѭʵϾ"㷨"ȡCRCУ
			if(crc32.CRC & 1)
				crc32.CRC = (crc32.CRC >> 1) ^ 0xEDB88320;
			else //0xEDB88320CRC-32ʽֵ
				crc32.CRC >>= 1;
		}
		crc32.CRC_32_Tbl[i] = crc32.CRC;
	}

	CRC32 = 0xffffffff; //óʼֵ
    for( i = 0; i < length; ++i)
    {
        CRC32 = crc32.CRC_32_Tbl[(CRC32^((unsigned char*)buffer)[i]) & 0xff] ^ (CRC32>>8);
    }

    //return CRC32;
	return CRC32^0xffffffff;
}
